from mag_tools.model.common.data_type import DataType


class DataFormat:
    def __init__(self, data_type, number_per_line = 1, right_justify = True, at_header = '', decimal_places=2, decimal_places_of_zero=1):
        """
        数据格式
        :param data_type: 数据类型
        :param number_per_line: 每行显示的数据个数
        :param right_justify: 是否右对齐
        :param at_header: 句首添加的字符串
        :param decimal_places: 小数位数
        :param decimal_places_of_zero: 小数为0时的小数位数
        """
        self.data_type = data_type
        self.number_per_line = number_per_line
        self.right_justify = right_justify
        self.at_header = at_header
        self.decimal_places = decimal_places if data_type == DataType.FLOAT else 0
        self.decimal_places_of_zero = decimal_places_of_zero if data_type == DataType.FLOAT else 0
        self.number_per_line = number_per_line
        self.right_justify = right_justify


from enum import Enum


class DataType(Enum):
    INTEGER = 1
    FLOAT = 2
    STRING = 3
    BOOLEAN = 4
    LIST = 5
    DICTIONARY = 6

    @classmethod
    def get_type(cls, text):
        # 尝试判断是否为整数
        try:
            int(text)
            return DataType.INTEGER
        except ValueError:
            pass

        # 尝试判断是否为浮点数
        try:
            float(text)
            return DataType.FLOAT
        except ValueError:
            pass

        # 判断是否为布尔值
        if text.lower() in ['true', 'false']:
            return DataType.BOOLEAN

        # 判断是否为列表
        if text.startswith('[') and text.endswith(']'):
            return DataType.LIST

        # 判断是否为字典
        if text.startswith('{') and text.endswith('}'):
            return DataType.DICTIONARY

        # 默认返回字符串类型
        return DataType.STRING


import re

class ListUtils:
    @staticmethod
    def split_segment(lines, keyword_map):
        text = "\n".join(lines)
        segments = {segment_name: [] for segment_name in keyword_map}
        segments["FIRST"] = []
        segments["END"] = []

        # 找到第一个有效段落的开始和最后一个有效段落的结束位置
        first_segment_start = None
        last_segment_end = None

        for segment_name, (begin_keyword, end_keyword) in keyword_map.items():
            pattern = rf"{segment_name}\n{begin_keyword}.*?\n{end_keyword}"
            matches = re.finditer(pattern, text, re.DOTALL)

            for match in matches:
                if first_segment_start is None:
                    first_segment_start = match.start()
                last_segment_end = match.end()

                block_lines = match.group().split("\n")
                segments[segment_name] = block_lines

        # 将第一个有效段落之前的内容放在 FIRST 段落
        if first_segment_start is not None:
            segments["FIRST"] = text[:first_segment_start].strip().split("\n")

        # 将最后一个有效段落之后的内容放在 END 段落
        if last_segment_end is not None:
            segments["END"] = text[last_segment_end:].strip().split("\n")

        return segments

if __name__ == '__main__':
    # 示例用法
    data = """MODELTYPE BlackOil
FIELD

GRID
##################################################
DIMENS
 5 2 1

BOX FIPNUM 1 5 1 2 1 1 = 2

PERMX
49.29276      162.25308      438.45926      492.32336      791.32867
704.17102      752.34912      622.96875      542.24493      471.45953

COPY PERMX  PERMY  1 5 1 2 1 1 
COPY PERMX  PERMZ  1 5 1 2 1 1

BOX  PERMZ  1 5 1 2 1 1  '*' 0.01

PORO
 5*0.087
 5*0.097

TOPS 10*9000.00

BOX TOPS   1  1  1 2  1  1  '='  9000.00
BOX TOPS   2  2  1 2  1  1  '='  9052.90

DXV
 5*300.0

DYV
 2*300.0

DZV
 20

#GRID END#########################################

WELL
##################################################

TEMPLATE
'MARKER' 'I' 'J' 'K1' 'K2' 'OUTLET' 'WI' 'OS' 'SATMAP' 'HX' 'HY' 'HZ' 'REQ' 'SKIN' 'LENGTH' 'RW' 'DEV' 'ROUGH' 'DCJ' 'DCN' 'XNJ' 'YNJ' /
WELSPECS
NAME 'INJE1'
''  24  25  11  11  NA  NA  SHUT  NA  0  0  0  NA  NA  NA  0.5  NA  NA  NA  1268.16  NA  NA  
''  24  25  11  15  NA  NA  OPEN  NA  0  0  DZ  NA  NA  NA  0.5  NA  NA  NA  0  NA  NA  

NAME 'PROD2'
''  5  1  2  2  NA  NA  OPEN  NA  0  0  DZ  NA  0  NA  0.5  0  0  NA  NA  NA  NA  
''  5  1  3  3  NA  NA  OPEN  NA  0  0  DZ  NA  0  NA  0.5  0  0  NA  NA  NA  NA  
''  5  1  4  4  NA  NA  OPEN  NA  0  0  DZ  NA  0  NA  0.5  0  0  NA  NA  NA  NA 
#WELL END#########################################

PROPS
##################################################
SWOF
#           Sw         Krw       Krow       Pcow(=Po-Pw)
       0.15109           0           1         400
       0.15123           0     0.99997      359.19
       0.15174           0     0.99993      257.92

#PROPS END########################################

SOLUTION
##################################################

EQUILPAR
# Ref_dep    Ref_p    GWC/OWC  GWC_pc/OWC_pc   dh
  9035       3600      9950        0.0         2
# GOC       GOC_pc
  8800        0.0    
PBVD
   5000        3600
   9000        3600

#SOLUTION END######################################

TUNE
TSTART  1990-01-01 
MINDT  0.1  MAXDT  31  DTINC  2.0  DTCUT  0.5  CHECKDX  
MAXDP  200  MAXDS  0  MAXDC  0  MBEPC  1E-4  MBEAVG  1E-6   
SOLVER  1034


RESTART

RPTSCHED
BINOUT SEPARATE NETONLY GEOM RPTONLY RSTBIN SOLVD 
POIL SOIL SGAS SWAT RS NOSTU  TECPLOT 
 /

RPTSUM
POIL 1 2 1 /
POIL AVG Reg 2 /
"""
    keyword_map = {
        "GRID": ("##################################################", "#GRID END#########################################"),
        "WELL": ("##################################################", "#WELL END#########################################"),
        "PROPS": ("##################################################", "#PROPS END########################################"),
        "SOLUTION": ("##################################################", "#SOLUTION END######################################")
    }

    _segments = ListUtils.split_segment(data.splitlines(), keyword_map)

    for segment_name, segment in _segments.items():
        print(f"Segment: {segment_name}")
        for line in segment:
            print(f"    {line}")
        print()



from enum import Enum

from mag_tools.utils.common.string_utils import StringUtils


class ControlType(Enum):
    """
        控件类型枚举
        枚举值为不包含前缀的控件类型名，如：ControlType.EDIT
    """

    #
    EDIT = "Edit"  # 文本框 TextBox
    DOC = "Document"  # 文档 Document
    #
    BUTTON = "Button"  # 按钮 Button
    SPLIT_BUTTON = "SplitButton" # 拆分按钮 SplitButton
    CHECKBOX = "CheckBox"  # 复选框 CheckBox
    RADIO = "RadioButton"  # 单选按钮 RadioButton
    #
    MENU_BAR = "MenuBar"  # 菜单栏 MenuBar
    MENU = "Menu"  # 菜单 Menu
    MENU_ITEM = "MenuItem"  # 菜单项 MenuItem
    #
    CONTEXT_MENU = "ContextMenu"  # 上下文菜单 ContextMenu
    #
    WINDOW = "Window"   # 主窗口 Main Window
    DIALOG = "Dialog"   # 对话框
    MESSAGE = "MessageBox"  # 消息框
    #
    LABEL = "Text"  # 标签 Label
    #
    LIST = "List"  # 列表框 ListBox
    LIST_VIEW = "ListView"  # 列表视图 ListView
    LIST_ITEM = "ListItem"  # 列表项， ListBox/ListView包含ListItem
    #
    COMBO_BOX = "ComboBox"  # 组合框 ComboBox
    #
    TREE = "Tree"  # 树视图 TreeView
    TREE_ITEM = "TreeItem"  # 树节点
    #
    TAB = "Tab"  # 选项卡 TabControl
    TAB_ITEM = "TabItem"    # Tab项
    GROUP_TAB = "GroupTab"  # 组TabItem
    #
    DATETIME = "SysDateTimePick32"   # 日期时间控件，类名为 SysDateTimePick32
    PROGRESS = "ProgressBar"  # 进度条 ProgressBar
    TITLE = "TitleBar"         # 标题栏 TitleBar
    SLIDER = "Slider"  # 滑块 Slider
    STATUS = "StatusBar"  # 状态条 StatusBar
    TOOL = "ToolBar"      # 工具栏 ToolBar
    GROUP = "Group" # 组Group
    PANEL = "Panel"  # Panel 分组和布局
    PANE = "Pane"  # Panel 分组框或面板

    @classmethod
    def get_by_value(cls, value):
        for type in cls:
            if type.value == value:
                return type
        return None

    @classmethod
    def type(cls, element):
        if element is None:
            return None

        type_name = StringUtils.get_after_keyword(element.tag_name, ".")
        return ControlType.get_by_value(type_name)

    def is_composite(self):
        return self in {ControlType.BUTTON, ControlType.SPLIT_BUTTON, ControlType.MENU, ControlType.COMBO_BOX,
                                 ControlType.LIST, ControlType.LIST_VIEW, ControlType.TREE, ControlType.PANE,
                                 ControlType.TOOL,
                                 ControlType.DATETIME, ControlType.WINDOW}

    @classmethod
    def get_name_by_chinese(cls, name_cn):
        _dict = {"文本框": "EDIT",
                 "文档": "DOC",
                 "按钮": "BUTTON",
                 "拆分按钮": "SPLIT_BUTTON",
                 "复选框": "CHECKBOX",
                 "单选按钮": "RADIO",
                 "菜单栏": "MENU_BAR",
                 "菜单": "MENU",
                 "菜单项": "MENU_ITEM",
                 "上下文菜单": "CONTEXT_MENU",
                 "主窗口": "WINDOW",
                 "对话框": "DIALOG",
                 "消息框": "MESSAGE",
                 "标签": "LABEL",
                 "列表框": "LIST",
                 "列表视图": "LIST_VIEW",
                 "列表项": "LIST_ITEM",
                 "组合框": "COMBO_BOX",
                 "树视图": "TREE",
                 "树节点": "TREE_ITEM",
                 "TAB": "TAB",
                 "TAB项": "TAB_ITEM",
                 "TAB组": "GROUP_TAB",
                 "工具栏": "TOOL",
                 "GROUP": "GROUP",
                 "PANE": "PANE",
                 "PANEL": "PANEL",
                 "标题栏": "TITLE",
                 "进度条": "PROGRESS",
                 "滑块": "SLIDER",
                 "状态条": "STATUS",
                 "日期时间": "DATETIME",
                 "组": "GROUP"}
        return _dict.get(name_cn)

    @classmethod
    def get_type_by_chinese(cls, name_ch):
        name = ControlType.get_name_by_chinese(name_ch)
        try:
            return cls[name]
        except KeyError:
            return None


import logging
import logging.config
import os

from mag_tools.model.common.log_type import LogType
from mag_tools.utils.common.string_utils import StringUtils


class Logger:
    __instance = None

    def __init__(self, app):
        app_name = os.path.splitext(os.path.basename(app))[0]

        current_dir = os.path.dirname(os.path.abspath(__file__))
        self.__root_dir = os.path.abspath(os.path.join(current_dir, '..\\..'))

        log_dir = os.path.join(self.__root_dir, "data", app_name, "logs")
        os.makedirs(log_dir, exist_ok=True)

        logging.config.fileConfig(f"{self.__root_dir}\\data\\logging.conf", defaults={'logdir': log_dir})

        self.root_logger = logging.getLogger()

        self.frame_logger = logging.getLogger('frame')
        self.__set_log_file(self.frame_logger, app_name)

        self.service_logger = logging.getLogger('service')
        self.__set_log_file(self.service_logger, app_name)

        self.performance_logger = logging.getLogger('performance')
        self.__set_log_file(self.performance_logger, app_name)

        Logger.__instance = self

    @staticmethod
    def initialize(app):
        if Logger.__instance is None:
            Logger(app)

    @staticmethod
    def debug(*args):
        if Logger.__instance is None:
            raise ValueError("Logger not initialized. Please call initialize first.")

        if len(args) == 1:
            message = args[0]
            Logger.__instance.__debug(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            Logger.__instance.__debug(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @staticmethod
    def info(*args):
        if Logger.__instance is None:
            raise ValueError("Logger not initialized. Please call initialize first.")

        if len(args) == 1:
            message = args[0]
            Logger.__instance.__info(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            Logger.__instance.__info(logger_type, message)
        elif len(args) == 3:
            logger_type, message, is_highlight = args
            if is_highlight:
                Logger.__instance.__info(logger_type, '*'*(StringUtils.get_print_width(message)+8))
                Logger.__instance.__info(logger_type, f'*** {message} ***')
                Logger.__instance.__info(logger_type, '*'*(StringUtils.get_print_width(message)+8))
            else:
                Logger.__instance.__info(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @staticmethod
    def error(*args):
        if Logger.__instance is None:
            raise ValueError("Logger not initialized. Please call initialize first.")

        if len(args) == 1:
            message = str(args[0]) if isinstance(args[0], Exception) else args[0]
            Logger.__instance.__error(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            message = str(message) if isinstance(message, Exception) else message
            Logger.__instance.__error(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    def __debug(self, logger_type, message):
        self.root_logger.debug(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.debug(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.debug(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.debug(message)

    def __info(self, logger_type, message):
        self.root_logger.info(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.info(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.info(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.info(message)

    def __error(self, logger_type, message):
        self.root_logger.error(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.error(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.error(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.error(message)

    def __set_log_file(self, logger, app_name):
        # handler = logger.handlers[0]
        # log_file = f"{self._data_dir}\\{app_name}\\logs\\{logger.name}.log"
        # handler.baseFilename = log_file
        # handler.stream = open(log_file, 'a')
        pass


import numpy as np


class ArrayUtils:
    @staticmethod
    def assign_array_2d(array2, i1, i2, j1, j2, value):
        """
        给数组指定区域的元素赋值
        :param array2: 二维数组
        :param i1: 行数1
        :param i2: 行数2
        :param j1: 列数1
        :param j2: 列数2
        :param value: 数值
        """
        for i in range(i1, i2 + 1):
            for j in range(j1, j2 + 1):
                if i < len(array2) and j < len(array2[i]):
                    array2[i][j]= value
                else:
                    raise IndexError("Index out of range for the provided 3D array dimensions.")
        return array2


    @staticmethod
    def multiply_array_2d(array2, i1, i2, j1, j2, factor):
        """
        给数组指定区域的元素乘以系数
        :param array2: 二维数组
        :param i1: 行数1
        :param i2: 行数2
        :param j1: 列数1
        :param j2: 列数2
        :param factor: 系数
        """
        for i in range(i1, i2 + 1):
            for j in range(j1, j2 + 1):
                if i < len(array2) and j < len(array2[i]):
                    array2[i][j] *= factor
                else:
                    raise IndexError("Index out of range for the provided 3D array dimensions.")
        return array2

    @staticmethod
    def add_array_2d(array2, i1, i2, j1, j2, value):
        """
        给数组指定区域的元素增加数值
        :param array2: 二维数组
        :param i1: 行数1
        :param i2: 行数2
        :param j1: 列数1
        :param j2: 列数2
        :param value: 数值
        """
        for i in range(i1, i2 + 1):
            for j in range(j1, j2 + 1):
                if i < len(array2) and j < len(array2[i]):
                    array2[i][j] += value
                else:
                    raise IndexError("Index out of range for the provided 3D array dimensions.")
        return array2

    @staticmethod
    def assign_array_3d(array3, i1, i2, j1, j2, k1, k2, value):
        """
        给数组指定区域的元素赋值
        :param array3: 三维数组
        :param i1: 层数1
        :param i2: 层数2
        :param j1: 行数1
        :param j2: 行数2
        :param k1: 列数1
        :param k2: 列数2
        :param value: 数值
        """
        for i in range(i1, i2 + 1):
            for j in range(j1, j2 + 1):
                for k in range(k1, k2 + 1):
                    if i < len(array3) and j < len(array3[i]) and k < len(array3[i][j]):
                        array3[i][j][k] = value
                    else:
                        raise IndexError("Index out of range for the provided 3D array dimensions.")
        return array3

    @staticmethod
    def multiply_array_3d(array3, i1, i2, j1, j2, k1, k2, factor):
        """
        给数组指定区域的元素乘以系数
        :param array3: 三维数组
        :param i1: 层数1
        :param i2: 层数2
        :param j1: 行数1
        :param j2: 行数2
        :param k1: 列数1
        :param k2: 列数2
        :param factor: 系数
        """
        for i in range(i1, i2 + 1):
            for j in range(j1, j2 + 1):
                for k in range(k1, k2 + 1):
                    if i < len(array3) and j < len(array3[i]) and k < len(array3[i][j]):
                        array3[i][j][k] *= factor
                    else:
                        raise IndexError("Index out of range for the provided 3D array dimensions.")
        return array3

    @staticmethod
    def add_array_3d(array3, i1, i2, j1, j2, k1, k2, value):
        """
        给数组指定区域的元素增加数值
        :param array3: 三维数组
        :param i1: 层数1
        :param i2: 层数2
        :param j1: 行数1
        :param j2: 行数2
        :param k1: 列数1
        :param k2: 列数2
        :param value: 数值
        """
        for i in range(i1, i2 + 1):
            for j in range(j1, j2 + 1):
                for k in range(k1, k2 + 1):
                    if i < len(array3) and j < len(array3[i]) and k < len(array3[i][j]):
                        array3[i][j][k] += value
                    else:
                        raise IndexError("Index out of range for the provided 3D array dimensions.")
        return array3


    @staticmethod
    def copy_array_3d(source_array, target_array, i1, i2, j1, j2, k1, k2):
        """
        复制源数组指定区域到目标数组
        :param source_array: 源数组
        :param target_array: 目标数组
        :param i1: 层数1
        :param i2: 层数2
        :param j1: 行数1
        :param j2: 行数2
        :param k1: 列数1
        :param k2: 列数2
        """
        for i in range(i1, i2 + 1):
            for j in range(j1, j2 + 1):
                for k in range(k1, k2 + 1):
                    if i < len(source_array) and j < len(source_array[i]) and k < len(source_array[i][j]):
                        target_array[i][j][k] = source_array[i][j][k]
                    else:
                        raise IndexError("Index out of range for the provided 3D array dimensions.")
        return target_array

    @staticmethod
    def init_array_3d(x, y, z):
        """
        初始化三维数据
        :x: 层数
        :y: 行数
        :z: 列数
        """
        return np.empty((x, y, z), dtype=object)

    @staticmethod
    def init_array_2d(x, y):
        """
        初始化二维数据
        :x: 行数
        :y: 列数
        """
        return np.empty((x, y), dtype=object)

if __name__ == '__main__':
    # 示例用法
    _array3 = [[[0 for _ in range(5)] for _ in range(4)] for _ in range(3)]

    _values = list(range(1, 19))  # 示例数据，共18个值
    ArrayUtils.assign_array_3d(_array3, 1, 2, 1, 3, 1, 3, _values)

    for layer in _array3:
        print(layer)

    ArrayUtils.multiply_array_3d(_array3, 1, 2, 1, 3, 1, 3, 2)
    print("\n数组乘以系数后：")
    for layer in _array3:
        print(layer)


    ArrayUtils.add_array_3d(_array3, 1, 2, 1, 3, 1, 3, 100.0)
    print("\n数组加数值后：")
    for layer in _array3:
        print(layer)